'Programm ......... : TWI-Slave LCD (TSL)
'
'Programminhalt ... : - TWI-Buskopplung eines LCD-Modul (2x 16 Zeichen)
'                       Im Datenbereich sind die Meldungen hinterlegt, die nach
'                       einem Datenempfang ber den Bus angezeigt werden.
'                     - LCD-Anzeige mit pulsweitenmodulierten Einstellmglich-
'                       keiten fr Kontrast und Hintergrundbeleuchtung (HGB)
'                     - ausbaufhiges Modul mit flexibler Adressierung
'
'Programmierer .... : Ingolf Bauer (ingolf.bauer@nexgo.de)
'Version .......... : V 1.1.83
'Datum ............ : 21.01.2012
'
'-------------------------------------------------------------------------------
'
'Simulator-Einstellmglichkeit
'-----------------------------
'$sim                                              'Wartezeit aus
'
' Achtung!
' Diese Zeile unbedingt auskommentieren, bevor der AVR programmiert wird, da
' sonst der AVR nicht korrekt arbeitet.
' Die Aktivierung ist nur fr den Simulator vorgesehen.
'
'-------------------------------------------------------------------------------
'
'Information
'-----------
'21.01.2012 - Nach einer neuen Programmierung ist es unbedingt erforderlich,
'             dass alle am TWI-Bus angeschlossenen AVR abgeschaltet und wieder
'             neu eingeschaltet werden.
'
'-------------------------------------------------------------------------------
'
'Versionsbersicht
'-----------------
'
'
'
'
'V 1.1.83  21.01.2012 - nderung .... : - Busbafrage in INT-Routine von Timer 1
'V 1.0.99  10.01.2012 - Information . : - Beginn der Softwareentwicklung
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
'Programmspeicher des MCs (Programm-Flashspeicher): V 1.1.83 - 37%
'
'-------------------------------------------------------------------------------
'
'Informationen zur Software
'--------------------------
' Inhalt                                          | Zeile | Korrekturwert
'------------------------------------------------------------------------
' 01. Allgemeine Informationen .................. |  115  |
'      Taktfrequenz ............................. |  117  |
'      Programmierung ........................... |  123  |
' 02. Abkrzungsbersicht ....................... |  130  |
' 03. Compilereinstellungen ..................... |  136  |
' 04. Funktionsbeschreibung ..................... |  159  |
'      Allgemeines .............................. |  161  |
'      Anzeige .................................. |  172  |
'      Adressierung ............................. |  179  |
'      Tastenbelegung ........................... |  189  |
' 05. Interrupt ................................. |  215  |
' 06. Einstellung der FUSE- und/oder LOOK-Bits .. |  220  |
' 07. Anschluschema fr den AVR (ATmega8) ...... |  233  |
' 08. I/O-Ports ................................. |  256  |
' 09. Steckverbinder ............................ |  333  |
' 10. Definition ................................ |  342  |
'      Schaltkreis .............................. |  344  |
'      Schnittstelle ............................ |  349  |
'      Compilereinstellungen .................... |  354  |
' 11. Deklaration ............................... |  362  |
'      Funktion ................................. |  364  |
'      Subrotine (Call) ......................... |  368  |
'      Feld ..................................... |  374  |
'      Flag ..................................... |  377  |
'      Variable ................................. |  384  |
'      Zhler ................................... |  398  |
' 12. Konfiguration ............................. |  411  |
'      I/O-Port ................................. |  413  |
'      Timer .................................... |  438  |
'      LCD ...................................... |  456  |
'      Interrupt ................................ |  466  |
' 13. Alias ..................................... |  478  |
' 14. Initialisierung ........................... |  484  |
' 20. Programm .................................. |  534  |
'      Test ..................................... |  537  |
'      Meldung .................................. |  541  |
'      Ausgabe der Meldung ...................... |  555  |
'      Tastenabfrage ............................ |  567  |
'      Ende Programm ............................ |  578  |
' 30. Unterprogramm ............................. |  584  |
'      Taste .................................... |  587  |
'      Bettigung kurz .......................... |  600  |
'      Bettigung lnger ........................ |  648  |
' 40. Interrupt ................................. |  707  |
'      Timer 0 .................................. |  710  |
'       PWM-Steuerung des LCD (Kontrast) ........ |  727  |
'       PWM-Steuerung des LCD (Helligkeit) ...... |  734  |
'      Timer 1 .................................. |  748  |
'       Prfung auf Datenempfang ................ |  764  |
'       Zeitzhler fr LCD-Beleuchtung .......... |  781  |
'       Zeitzhler fr LCD-Kontarststeuerung .... |  789  |
'       Lebensbit ............................... |  794  |
' 50. Unterprogramm-Call ........................ |  808  |
'      Hintergrundbeleuchtung ................... |  811  |
'      Aktivierung des Moduls ................... |  823  |
' 60. Funktion .................................. |  837  |
'      Datenempfang ............................. |  840  |
' 70. Datenbereich .............................. |  864  |
'
'
'-------------------------------------------------------------------------------
'
'01. Allgemeine Informationen
'----------------------------
'Taktfrequenz
' Als Taktfrequenz wir eine Quarzfrequenz von 16,0 MHz verwendet.
' Werden die Bedingungen (die der TWI-Bus erfordert) eingehalten, kann auch
' mit einer geringeren Frequenz getaktet werden. Die Timer-Einstellungen sind
' dann anzupassen.
'
'Programmierung
' Die Programmierung der MC kann extern oder ber die vorhandene Schnittstelle
' erfolgen.
' Zur Programmierung wurde das Modul "USBasp" verwendet, welches ber eine
' USB-Schnittstelle angeschlossen wird. Es kann direkt ber eine, in BASCOM
' integrierte Einstellung angesprochen werden.
'
'02. Abkrzungsbersicht
'-----------------------
' HGB ... : Hintergrundbeleuchtung
' PWM ... : Pulsweitenmodulation
' TWI ... : Two-wire Serial Interface (entspricht technisch dem I2C-Bus)

'03. Compilereinstellungen
'-------------------------
' Hinweise fr ATmega8/ATmega16 mit 1024 Byte SRAM
' Quelle:
' //halvar.at/elektronik/kleiner_bascom_avr_kurs/speicher_hwstack_swstack_frame/
'
' HW-Stack    - Empfehlung: 40 Byte bei INT, sonst 10 Byte ausreichend
'  32 Byte fr INT + je GOSUB 2 Byte, wenn bei INT ein GOSUB erfolgt
'  (bei Verschachtelung ... + n x 2 Byte)
'
' SW-Stack    - Empfehlung: 32 Byte (recht komplexe Aufrufe werden abgedeckt)
'   2 Byte fr jede bergebene Variable einer GOSUB-Routine + 2 Byte fr jede
'   LOCAL-Variable innerhalb einer GOSUB
'   (bei Verschachtelung ... n x 2 Byte)
'
' Frame-Space - Empfehlung: 60 Byte sollten alles abdecken
'                           (bei wenig Speicherverkleinern)
'  Beispiel:
'   2 x BYTE        =  2 Byte (erster und zweiter Parameter)
'   1 x WORD        =  2 Byte (dritte Parameter)
'   1 x STRING * 10 = 11 Byte (inkl. Abschlussbyte; vierter Parameter)
'   3 x BYTE        =  3 Byte (lokale Variablen)
'
'04. Funktionsbeschreibung
'-------------------------
' Allgemeines
' -----------
'  Stromversorgung
'  Sie erfolgt im normalen Betrieb ber die Schnittstelle, ber die auch der
'  TWI-Bus und die Programmiersschnittstelle angeschlossen werden und muss
'  stabilisiert sein sowie einen Wert von +5 V haben. Es ist auf dem Modul kei-
'  ne Sicherung bei Kurzschluss oder Verpolung vorhanden.
'  Wird ein Programmiergert eingesetzt, dass diese Spannung nicht liefert, be-
'  steht ber einen zweipoligen Steckverbinder die Mglichkeit, diese extern
'  einzuspeisen.
'
' Anzeige
' -------
'  Neben dem LC-Display wird ein Ausgabekanal fr die Ansteuerung einer LED
'  eingesetzt.
'  Beim Programmstart blinkt sie dreimal kurz hintereinander. Spter signali-
'  siert ein kurzes Blitzen die Betriebsbereitschaft des Moduls ("Lebensbit").

' Adressierung
' ------------
'  Bei Modulen, die ber ein Bussystem verbunden sind, muss jeder Teilnehmer
'  eine eindeutige Adresse erhalten. Die Basisadresse des TSL ist mit &H50
'  festgelegt. Diese Adresse kann aber im Programm verndert werden. Auerdem
'  ist es durch das Setzen von Jumpern an den Kanlne PB.0-PB.2 mglich, diese
'  Adresse in Zweierschritten zu erhhen. Damit ist ein Adressraum von &H50 bis
'  &H5E mglich.
'  Beim Start wird innerhalb der "Begrungsbildschirm" die Adresse angezeigt.
'
' Tastenbelegung
' --------------
'  Hinweis:
'  Softwaremig wird unterschieden, ob eine Taste kurz (-k) gedrckt oder
'  lnger gehalten wird (-l).
'
'  Taste 1-k: Die Hintergrundbeleuchtung wird fr eine Zeitdauer, die im Pro-
'             gramm eingestellt ist, eingeschaltet.
'  Taste 1-l: Alle Medungstexte, die im DATA-Bereich definiert sind, werden
'             nacheinander zur Anzeige gebracht.
'
'  Taste 2-k: Der Kontrast wird um einen Schritt bis zum Erreichen eines maxi-
'             malen Wertes verndert. Ist zwischen den Bettigungen eine etwas
'             grere Pause, erfolgt eine Richtungsumkehr. Das bedeutet einen
'             Wechsel von "dunkler" nach "hell" bzw. umgekehrt.
'             Der eingestellte Wert wird im EEprom gespeichert, von dort bei
'             einem Neustart eingelesen und verwendet.
'
'  Taste 2-l: (nicht mit einer Funktion belegt)
'
'  Taste 3-k: Hiermit erfolgt der Aufruf einer Test-Anzeige.
'  Taste 3-l: Die Hintergrundbeleuchtung ndert sich kontinuierlich von "dunkel"
'             nach "hell" und umgekehrt. Wird die Taste losgelassen, wird der
'             eingestellte Wert im EEprom gespeichert und von dort bei einem
'             Neustart verwendet.
'
'05. Interrupt
'-------------
' Timer 0:  8 Bit, Interrupt bei berlauf (PWM-Steuerung)
' Timer 1: 16 Bit, Interrupt bei berlauf (Zeitbasis)
'
'06. Einstellung der FUSE- und/oder LOOK-Bits
'--------------------------------------------
' Fr die Einstellung der Fuse- und Lookbits ist der Bauanleitung ein "screen-
' shot" (Programmer: USBasp) beigelegt.

' /// Achtung ///
' CKSEL
'  Wird CKSEL3= 0 gesetzt, arbeitet der MC nur noch mit einem externen RC-
'  Generator [f= 1/(3RC); C>= 22p].
'
' PC6
'  Fr diesen Kanal muss die Betriebsart "Reset" eingestellt werden.
'
'07. Anschluschema fr den AVR (ATmega8)
'----------------------------------------
'
'                    --------------------|_|-------------------
' [~]    /Reset ---> | 1 [/Reset]          [SLC, ADC5, PC5] 28| <--> SLC    [B]
' [0]   LCD_E   <--- | 2 [PD0, RXD]        [SDA, ADC4, PC4] 27| <--> SDA    [B]
' [0]   LCD_RS  <--- | 3 [PD1, TXD]             [ADC3, PC3] 26| ---> LED    [0]
' [0]   LCD_DB4 <--- | 4 [PD2, INT0]            [ADC2, PC2] 25| <--- T_3    [1]
' [0]   LCD_DB5 <--- | 5 [PD3, INT1]            [ADC1, PC1] 24| <--- T_2    [1]
' [0]   LCD_DB6 <--- | 6 [PD4, T0]              [ADC0, PC0] 23| <--- T_1    [1]
'           P5V ==== | 7 [VCC]                       [AGND] 22| ==== GND
'           GND ==== | 8 [GND]                       [AREF] 21| ==== P5V
'            QZ ++++ | 9 [XT1]                       [AVCC] 20| ==== P5V
'            QZ ++++ |10 [XT2]                    [SCK,PB5] 19| <--- SCK    [~]
' [0]   LCD_DB7 <--- |11 [PD5, T1]               [MISO,PB4] 18| ---> MISO   [~]
' [0]   LCD_KON <--- |12 [PD6, AIN0]             [MOSI,PB3] 17| <--- MOSI   [~]
' [0]   LCD_HGB <--- |13 [PD7, AIN1]             [/SS, PB2] 16| <--- ADR_2  [1]
' [1]     ADR_0 ---> |14 [PB0, ICP]              [OC1, PB1] 15| <--- ADR_1  [1]
'                    ------------------------------------------
'
' Werte in []: Einstellungen nach dem Programmstart
'  ~: Impuls   0: aus   1: ein   B: Bussystem
'
'08. I/O-Ports
'-------------
' /Reset  : Resetfunktion
' ADR_    : Adressbit
' LCD_    : LCD-Modul
' LED     : Anschluss fr LED
' SDA     : Busanschluss (Daten)
' SLC     : Busanschluss (Takt)
' T_      : Taster
'
' _DBx    : Datenbit (x: 4, 5, 6, 7)
' _E      : Enable
' _HGB    : Hintergrundbeleuchtung (pulsweitenmoduliert)
' _KON    : Kontrasteinstellung    (pulsweitenmoduliert)
' _RS     : Steuerung
' _x      : Nummer fr Adressbit, Taster (x: 1, 2, 3)
'
' P5V     : Stromversorgung P 5V
' GND     : Stromversorgung Masse
'
' MISO    : Programmieranschlu
' MOSI    : Programmieranschlu
' QZ      : Anschlu Quarz
' SCK     : Programmieranschlu

'           Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' -   : SV  P5V ........ | ............. | Stromversorgung .............. |  7
' -   : SV  GND ........ | ............. | Stromversorgung .............. |  8
' -   : SV  P5V ........ | ............. | Stromversorgung .............. | 20
' -   : SV  GND ........ | ............. | Stromversorgung .............. | 22
' -   : SV  P5V ........ | ............. | Referenzspannung ............. | 21
' -   : PRG /RESET ..... | ............. | Programmierung/Reset des MCs . |  1
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... |  9
' -   : QZ  Frequenz ... | ............. | Frequenznormal mit Quarz ..... | 10
'
'Port B     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  Adresse 0 .. | ADR_0 ....... | Codierung der Slave-Adresse .. | 14
' - .1: DE  Adresse 1 .. | ADR_1 ....... | Codierung der Slave-Adresse .. | 15
' - .2: DE  Adresse 2 .. | ADR_2 ....... | Codierung der Slave-Adresse .. | 16
' - .3: PRG MOSI ....... | MOSI ........ | Programmierung ............... | 17
' - .4: PRG MISO ....... | MISO ........ | Programmierung ............... | 18
' - .5: PRG SCK ........ | SCK ......... | Programmierung ............... | 19
' - nc: ... ............ | ............. | .............................. | nc
' - nc: ... ............ | ............. | .............................. | nc
'
'Port C     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DE  diverse .... | T_1 ......... | HGB (Ein/Aus), Meldungsausgabe | 23
' - .1: DE  diverse .... | T_2 ......... | Kontrast ..................... | 24
' - .2: DE  diverse .... | T_3 ......... | LCD-Test, HGB-Helligkeit ..... | 25
' - .3: DA  Anzeige .... | LED ......... | Anzeige (Start, Lebensbit) ... | 26
' - .4: B   Daten (Bus). | SDA ......... | TWI-Bussystem ................ | 27
' - .5: B   Takt  (Bus . | SLC ......... | TWI-Bussystem ................ | 28
' - nc: ... ............ | ............. | .............................. | nc
' - nc: ... ............ | ............. | .............................. | nc
'
'Port D     Funktion     | Signal        | Erluterung                    | Pin
' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' - .0: DA  Enable ..... | LCD_E ....... | LCD-Aktivierungssignal ....... |  2
' - .1: DA  Steuerung .. | LCD_RS ...... | LCD-Steuersignal ............. |  3
' - .2: DA  Datenbit 4 . | LCD_DB4 ..... | LCD-Datenbit 4 ............... |  4
' - .3: DA  Datenbit 5 . | LCD_DB5 ..... | LCD-Datenbit 5 ............... |  5
' - .4: DA  Datenbit 6 . | LCD_DB6 ..... | LCD-Datenbit 6 ............... |  6
' - .5: DA  Datenbit 7 . | LCD_DB7 ..... | LCD-Datenbit 7 ............... | 11
' - .6: DA  Kontrast ... | LCD_KON ..... | Kontrasteinstellung    (PWM) . | 12
' - .7: DA  Beleuchtung  | LCD_HGB ..... | Hintergrundbeleuchtung (PWM) . | 13
'
'Erluterung
' B  : Bussystem
' DA : digitaler Ausgang
' DE : digitaler Eingang
' PRG: Programmierung
' QZ : Quarzanschluss
' SV : Stromverssorgung
'
'09. Steckverbinder
'------------------
' X1 : Bussystem, Programmer, Stromversorgung   Wannen-Steckverbinder (10-polig)
' X2 : Stromversorgung (zustzlich)             LP-Steckverbinder     ( 2-polig)
' X3 : Einstellung der Slave-Adress             LP-Steckverbinder     ( 6-polig)
' X4 : Eingabe (Anschluss der Taster, extern)   LP-Steckverbinder     ( 4-polig)
'
'-------------------------------------------------------------------------------
'
'10. Definition
'--------------
'Schaltkreis
'-----------
   $regfile = "m8def.dat"                         'Prozessor ....... : ATmega8
   $crystal = 16000000                            'Quarzfrequenz ... : 16,0 MHz

'Schnittstelle
'-------------
' ggf. fr eine Ausgabe im Simulator-Feld (UART)
' $baud = 9600                                   'Baudrate ........ : 9600
                                                  '9600/19200/38400/57600/115200
'Compilereinstellungen
'---------------------
   $hwstack = 40                                  'Hardware Stack
   $swstack = 32                                  'Software Stack
   $framesize = 60                                'Frame Space
'
'-----------------------------------------------------------------------------
'
'11. Deklaration
'---------------
'Funktion
' Slave-Datenempfang
   Declare Function Twi_wait() As Byte

'Subrotine (Call)
' Slave-Initialisierung
   Declare Sub Slave_init
' LCD-Ausgabe
   Declare Sub Lcd_hgb(byval T_hgb As Byte)

'Feld
   Dim Slv_wert(5)as Byte                         'Array (5)

'Flag
   Dim F_hel As Bit                               'Richtung Helligkeit
   Dim F_hgb As Bit                               'LC-Beleuchtung
   Dim F_kon As Bit                               'Richtung Kontrast
   Dim F_lb As Bit                                'Lebensbit
   Dim F_taste As Bit                             'Taste

'Variable
   Dim Pwm_hgb As Byte                            'LCD: Helligkeit
   Dim Pwm_kon As Byte                            'LCD: Kontrastwert

   Dim Slv_adr As Byte                            'Slave_Adresse
   Dim Slv_arr As Byte                            'Feldelement
   Dim Slv_dat As Byte                            'Slave_Daten
   Dim Slv_msg As String * 32                     'Meldung
   Dim Slv_msg_old As String * 32                 'Meldung_alt
   Dim Slv_sta As Byte                            'Slave_Status

   Dim Taste As Byte                              'Taste
   Dim Taste_old As Byte                          'Taste (Merker)

'Zhler
   Dim Hz01 As Byte                               'Hilfszhler 01

   Dim Z_hgb As Byte                              'LCD-Beleuchtung
   Dim Z_lb As Byte                               'Zeit T1 (Lebensbit)
   Dim Z_kon As Byte                              'Wartezeit Richtung
   Dim Z_t0a As Byte                              'Zeit T0a
   Dim Z_t0b As Byte                              'Zeit T0b
   Dim Z_t1 As Byte                               'Zeit T1
   Dim Z_taste As Byte                            'Lnge Tastendruck
'
'-------------------------------------------------------------------------------
'
'12. Konfiguration
'-----------------
'I/O-Port
'--------
'=Port B
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrb = &B0000_0000                             'PB0-7 Ein
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portb = &B0000_0111                            'PB0-2 aktiv, 3-7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port C
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrc = &B0000_1000                             'PC0-2, 4-7 Ein, PC3 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portc = &B0000_0111                            'PC0-2 aktiv, PC4-7 passiv
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'=Port D
'-Datenrichtung (Ein [0]: Eingang, Aus [1]: Ausgang)
   Ddrd = &B1111_1111                             'PD0-7 Aus
'-Pullup-Widerstand (passiv [0], aktiv [1])
   Portd = &B0011_1111                            'PD0-5 aktiv, PD6-7 passiv
'Achtung - nicht fr folgende Ausgnge, da PWM-moduliert:
'- PD6 : Kontrast (positive Spannung => Kontrast reduziert)
'- PD7 : LCD-Hintergrundbeleuchtung (HGB)
'
'-------------------------------------------------------------------------------
'
'Timer
'-----
'Normal-Konfiguration
'Die Timerinterrupts 0/1 werden beim jeweiligen Timerberlauf ausgelst.
'                                   PSC  TVE   T         f
' T0 = 1/ Quarz* Teiler (1/ 16 MHz* 256* 193 =  1,00 ms; 992 Hz)
' T1 = 1/ Quarz* Teiler (1/ 16 MHz* 256* 625 = 10,00 ms; 100 Hz)

'Timer0 voreinstellen auf Wert: 256-TVE
   Config Timer0 = Timer , Prescale = 256         'Konfiguration
      Timer0 = 193                                'Voreinstellung

'Timer1 voreinstellen auf Wert: 65536-TVE
   Config Timer1 = Timer , Prescale = 256         'Konfiguration
      Timer1 = 64911                              'Voreinstellung
'
'-------------------------------------------------------------------------------
'
'LCD
'---
'-LCD-Typ ..... : 2 Zeilen, 16 Zeichen
   Config Lcd = 16 * 2
   Config Lcdpin = Pin , Db4 = Portd.2 , Db5 = Portd.3 , Db6 = Portd.4 , _
          Db7 = Portd.5 , E = Portd.0 , Rs = Portd.1
   Cls                                            'Anzeige lschen

'-------------------------------------------------------------------------------
'
'Interrupt
'---------
   Enable Timer0                                  'Timer 0: Interrupt
      On Timer0 Int_tim0                          'Timer 0: berlauf

   Enable Timer1                                  'Timer 1: Interrupt
      On Timer1 Int_tim1                          'Timer 1: berlauf

   Enable Interrupts                              'Interrupts: global zulassen
'
'-------------------------------------------------------------------------------
'
'13. Alias
'---------
' (nicht verwendet)
'
'-------------------------------------------------------------------------------
'
'14. Initialisierung
'-------------------
'TWI (Basisadresse &H50)
   Slv_adr = Pinb And &B0000_0111                 'Abfrage Jumper
   Rotate Slv_adr , Left                          'Normierung auf Bit 1
   Slv_adr = Slv_adr Xor &B0000_1110
'Adressbereich: &H50 ... &H5E (nur geraden Adressen mglich)
   Slv_adr = Slv_adr + &H50                       'Slave_Adresse festlegen
   Slv_dat = 0                                    'Slave_Datenbyte: 0
   Slv_arr = 0                                    'Zhler: Feldelement

   Call Slave_init                                'TWI aktivieren

'LCD
' Kontrast
   Readeeprom Pwm_kon , &H01
      If Pwm_kon > 20 Then Pwm_kon = 20           'Erststart: Wert 20
' Helligkeit (Hintergrundbeleuchtung)
   Readeeprom Pwm_hgb , &H02
      If Pwm_hgb > 20 Then Pwm_hgb = 20           'Erststart: Wert 20

'Blinken beim Programmstart (nach einem Reset)
'Lebensbit nach Startroutine deaktivieren
   F_lb = 0

   For Hz01 = 1 To 6
      Portc.3 = Not Portc.3                       'LED_gn (3x)
      Waitms 300
   Next Hz01

'LCD-Anzeige: Ausgabe der Softwareinformation (Version der Firmware)
   Cursor Off
   Call Lcd_hgb(200)                              'LCD-Beleuchtung: 20 s

   Lcd "Amatronik - LCD "                         'LCD-Ausgabe, Zeile 1
   Lowerline
   Lcd "Version: 1.1.83 "                         'LCD-Ausgabe, Zeile 2
      Wait 2                                      'Anzeigezeit
   Lowerline
   Lcd "  Slave-ADR &H"
   Lcd Hex(slv_adr)                               'Abzeige: Slave-Adresse
      Wait 2                                      'Anzeigezeit
   Cls                                            'Anzeige lschen

'nach Startroutine aktivieren:
   F_lb = 1                                       'Lebensbit
   F_kon = 1                                      'Kontrast (Richtung)
'
'-------------------------------------------------------------------------------
'
'20. Programm
'------------
Do
'Test
'----
'   Slv_msg = "<Meldung>       >"

'Meldung
'-------
'Dem ber die Schnittstelle empfangenen Code wird eine Meldung aus dem DATA-
'Bereich zugeordnet.
' 1. Feldelement: Anzahl der zu bertragenden Bytes
' 2. Feldelement: Meldungscode
   If Slv_wert(2) <= 9 Then                       'Meldung im Data-Bereich?
      If Slv_wert(2) <> 0 Then                    'Meldungs-Code vorhanden
         Slv_msg = Lookupstr(slv_wert(2) , Dat_1) 'Meldung aus dem DATA-Bereich
      Else                                        'zuordnen
         Slv_msg = ">"                            'Startbildschirm
      End If
   End If

'Ausgabe der Meldung
'-------------------
'Anzeige nur, wenn sich der Text gendert hat.
   If Slv_msg <> Slv_msg_old Then                 'auf Vernderung prfen
      Call Lcd_hgb(30)                            'LCD-Beleuchtung: 3 s
      Cls                                         'LCD: lschen
      Upperline : Lcd = Left(slv_msg , 16)        'LCD: Zeile 1, Zeichen  1-16
      Lowerline : Lcd = Mid(slv_msg , 17)         'LCD: Zeile 2, Zeichen 17-32

      Slv_msg_old = Slv_msg                       'Meldung speichern
   End If

'Tastenabfrage
'-------------
'Der Aufruf wird ber einen Timer-Interrupt gesteuert.
   If F_taste = 1 Then                            'Flag abfragen
      Gosub Up_taste
      F_taste = 0                                 'Reset Flag
   End If

'Ende Programmschleife
Loop

'Ende Programm
End

'===============================================================================

'-------------------------------------------------------------------------------
'30. === U N T E R P R O G R A M M =============================================
'-------------------------------------------------------------------------------

'Taste
'-----
Up_taste:
' Prfung, ob ein kurzer Druck oder die Taste lnger gehalten wird
   Taste_old = Taste                              'gedrckte Taste merken
   Taste = Pinc And &B0000_0111                   'Kanal 0-2 abfragen

   If Taste = 6 Or Taste = 5 Or Taste = 3 Then
      If Z_taste < 100 Then
         Incr Z_taste                             'Zeitzhler
      End If                                      'bei einer gedrckten Taste
   End If

'Taste wurde nach einer kurzen Bettigung losgelassen
'++++++++++++++++++++++++++++++++++++++++++++++++++++
   If Taste <> Taste_old And Z_taste > 2 And Z_taste <= 20 Then
      Select Case Taste_old                       'gedrckte Taste

'Taste 1 - kurz (000_0110)
'-------------------------
'Beleuchtung: einschalten (Zeitberwachung schaltet nach 20 s aus)
         Case 6
            Call Lcd_hgb(200)                     'LCD-Beleuchtung: 20 s

'Taste 2 - kurz (000_0101)
'-------------------------
'Kontrast: keine Richtungsumkehr, wenn innerhalb einer Zeit t eine Tastenbet-
'tigung stattgefunden hat.
         Case 5
            If Z_kon = 0 Then
               F_kon = Not F_kon
            End If

            Call Lcd_hgb(50)                      'LCD-Beleuchtung: 5 s
            Z_kon = 20                            'Wartezeit

'Fr die Kontrasteinstellung sind maximal 20 Stufen festgelegt.
'Mglich wren 50, aber dies ndert nichts mehr am Kontrast.
            If F_kon = 0 And Pwm_kon < 20 Then
               Incr Pwm_kon
            End If
            If F_kon = 1 And Pwm_kon > 0 Then
               Decr Pwm_kon
            End If

            Writeeeprom Pwm_kon , &H01            'Wert im EEprom speichern

'Taste 3 - kurz (000_0011)
'-------------------------
'LCD: Testmodus
         Case 3
            Call Lcd_hgb(100)                     'LCD-Beleuchtung: 10 s
            Cls : Lcd "LCD - Test" : Wait 2 : Cls
            Lcd "X-X-X-X-X-X-X-X-"                'LCD_Zeile 1: <Testzeile>
            Lowerline
            Lcd "-X-X-X-X-X-X-X-X"                'LCD_Zeile 2: <Testzeile>
            Wait 3 : Cls                          'warten und lschen
            Slv_msg_old = " "                     'Anzeige aktualisieren
      End Select
   End If

'Taste gedrckt bei bereits lngere Bettigungszeit
'++++++++++++++++++++++++++++++++++++++++++++++++++
   If Taste = Taste_old And Z_taste > 20 Then
      Select Case Taste_old                       'gedrckte Taste

'Taste 1 - lang (000_0110)
'-------------------------
'Anzeige: Ausgabe aller im Bereich "DATA" hinterlegten Meldungen
         Case 6
            Call Lcd_hgb(50)                      'LCD-Beleuchtung: 5 s
'Achtung!
'--------
'Sollen mehr Daten gelesen werden, als im DATA-Bereich eingetragen sind, strzt
'das Programm im MC ab.
            Cls : Lcd "Meldungen"                 'Ausgabe
            Lowerline : Lcd " (Bereich DATA) " : Wait 3
            For Hz01 = 0 To 9
               Call Lcd_hgb(10)                   'LCD-Beleuchtung: 1 s
               Cls : Lcd Lookupstr(hz01 , Dat_1) : Waitms 500
            Next Hz01
'LC-Beleuchtung noch kurz anlassen
            Call Lcd_hgb(50)                      'LCD-Beleuchtung: 5 s
            Cls : Lcd "Ende der Ausgabe" : Wait 2
            Slv_msg_old = " "                     'Anzeige aktualisieren
            Cls

'Taste 2 - lang (000_0101)
'-------------------------
'+ + + Funktion noch frei definierbar + + +
         Case 5

'Taste 3 - lang (000_0011)
'-------------------------
'LCD: Helligkeit der Hintergrundbeleuchtung
         Case 3
            Call Lcd_hgb(50)                      'LCD-Beleuchtung: 5 s

            If F_hel = 0 And Pwm_hgb < 20 Then
               Incr Pwm_hgb : Waitms 300
               If Pwm_hgb = 20 Then F_hel = 1     'Richtungsumkehr: HGB dunkler
            Else
               Decr Pwm_hgb : Waitms 300
               If Pwm_hgb = 1 Then F_hel = 0      'Richtungsumkehr: HGB heller
            End If

            Writeeeprom Pwm_hgb , &H02            'Wert im EEprom speichern
            Slv_msg_old = " "                     'Anzeige aktualisieren

      End Select
   End If

'Zeitzhler fr die Lnge der Bettigungszeit zurcksetzen
   If Taste <> Taste_old Then
      Z_taste = 0
    End If

Return

'-------------------------------------------------------------------------------
'40. === I N T E R R U P T R O U T I N E =======================================
'-------------------------------------------------------------------------------

'Timer 0
'-------
Int_tim0:
'T0 = 1/ Quarz* Teiler (1/ 16 MHz* 256* 193 =  1,00 ms; 992 Hz)
'Interrupt: Timer 0 (8 Bit), alle 1 ms
'Voreinstellung des Zhlers
   Timer0 = 193                                   'Voreinstellung (1 ms)
   Incr Z_t0a : Incr Z_t0b                        'Zeitbasis (A, B)

   If Z_t0a = 50 Then                             'Zhler A: Kontrast
      Z_t0a = 0
   End If

   If Z_t0b = 20 Then                             'Zhler B: LCD-Beleuchtung
      Z_t0b = 0
   End If

'PWM-Steuerung des LCD (Kontrast)
'--------------------------------
'Eine lngere Einschaltzeit bedeutet eine hhere Spannung und damit einen ge-
'ringeren Kontrast des LCD.
   If Z_t0a = 0 Then Portd.6 = 1                  'LCD-Kontrast: Ein
   If Z_t0a = Pwm_kon Then Portd.6 = 0            'LCD-Kontrast: Aus

'PWM-Steuerung des LCD (Helligkeit)
'----------------------------------
'Eine lngere Einschaltzeit bedeutet eine hellere Hintergrundbeleuchtung.
   If F_hgb = 1 Then
      If Z_t0b = 0 Then Portd.7 = 1               'LCD-Beleuchtung: Ein
      If Z_t0b = Pwm_hgb Then Portd.7 = 0         'LCD-Beleuchtung: Aus
   Else
      Portd.7 = 0                                 'LCD-Beleuchtung: Aus
   End If

Return
'
'-------------------------------------------------------------------------------
'
'Timer 1
'-------
Int_tim1:
'T1 = 1/ Quarz* Teiler (1/ 16 MHz* 256* 625 = 10,00 ms; 100 Hz)
'Interrupt: Timer 1 (16 Bit), alle 10 ms

'Voreinstellung des Zhlers
   Timer1 = 64911                                 'Voreinstellung (10 ms)

'Tastaturabfrage
   F_taste = 1                                    'Flag setzen

'Zeitzhler
   Incr Z_t1                                      'Zhler: Zeit (allgemein)
   Incr Z_lb                                      'Zhler: Lebensbit

'Prfung auf Datenempfang (TWI-Bus)
'----------------------------------
   Slv_sta = Twi_wait()                           'TWINT prfen, ob gesetzt ist

   If Slv_sta = &H80 Then
      Incr Slv_arr                                'Zhler: Feldelement
         Slv_wert(slv_arr) = Slv_dat              'Daten ins Feld eintragen
'Die Anzahl der zu bertragenden Bytes sind in der Feldvariable (1) des Arrays
'festgelegt.
'Rcksetzen des Zhlers (Anzahl der Feldelemente)
         If Slv_arr = Slv_wert(1) Then            'alle Ports bertragen
            Slv_arr = 0                           'Voreinstellung
         End If
   End If

   If Z_t1 = 10 Then                              'Rcksetzen alle 100 ms
      Z_t1 = 0
'Zeitzhler fr LCD-Beleuchtung
'------------------------------
      If Z_hgb > 0 Then
         Decr Z_hgb                               'Zeitzhler: LCD-Beleuchtung
      Else
         F_hgb = 0
      End If

'Zeitzhler fr LCD-Kontraststeuerung
'------------------------------------
      If Z_kon > 0 Then Decr Z_kon                'Zeitzhler: LCD-Kontrast
   End If

'Lebensbit
'---------
   If F_lb = 1 Then                               'erst nach der Startroutine
      Select Case Z_lb
         Case 0
            Portc.3 = 1                           'LED_gn Ein
         Case 10
            Portc.3 = 0                           'LED_gn Aus
      End Select
   End If

Return

'-------------------------------------------------------------------------------
'50. === U N T E R P R O G R A M M - C A L L ===================================
'-------------------------------------------------------------------------------

'Hintergrundbeleuchtung
'----------------------
'LCD-Modul: Hintergrundbeleuchtung einschalten (Zeitdauer in ms wird bergeben)
Sub Lcd_hgb(byval T_hgb As Byte )
'Zhler initialisieren
  Z_hgb = T_hgb                                   'LCD-Beleuchtungszeit
      F_hgb = 1

End Sub
'
'-------------------------------------------------------------------------------
'
'Aktivierung des Moduls
'----------------------
'Modul als als Slave am TWI-Bus aktivieren
Sub Slave_init
'Einstellung der Register
    Twsr = 0                                      'Status und Prescaler auf 0
    Twdr = &HFF                                   'default
    Twcr = &B00000100                             'erstmal nur TWI aktivieren
    Twar = Slv_adr                                'Slave_Adresse setzen
    Twcr = &B01000100                             'dann ACK einschalten

End Sub

'-------------------------------------------------------------------------------
'60. === F U N K T I O N =======================================================
'-------------------------------------------------------------------------------

'Datenempfang
'------------
Function Twi_wait() As Byte
'Prfung, ob was gekommen ist; Status zurckgeben; Daten in globale Variable

'Deklaration einer lokalen Variable
   Local Twcr_local As Byte

'Routine
   Twi_wait = &HF8                                'warten bis TWINT gesetzt ist
   Twcr_local = Twcr And &H80                     'Bit 7 von Controlregister

   If Twcr_local = &H80 Then
      Slv_dat = Twdr                              'Daten sichern
      Twi_wait = Twsr And &HF8                    'Status zurckgeben

'TWINT muss immer gelscht werden, damit es auf dem Bus weiter geht
'ACK: Senden dieses Besttigungsbits zulassen
      Twcr = Twcr Or &B10000000                   'TWINT lschen, ACK erzeugen
   End If

End Function

'-------------------------------------------------------------------------------
'70. === D A T E N B E R E I C H ===============================================
'-------------------------------------------------------------------------------

'Wird ein Code zur Ausgabe einer Meldung ber den TWI-Bus empfangen, erfolgt die
'Ausgabe der Meldung.
'Die Eintrge sind auf maximal 32 Zeichen (2 Zeilen mit je 16 Zeichen begrenzt).

'Achtung!
'--------
'Werden mehr Daten gelesen, als im DATA-Bereich eingetragen sind, strzt das
'Programm im MC ab.
'Dieser Bereich kann individuell angepasst werden.

Dat_1:
   Data "-----------" , "Meldung 001"
   Data "Meldung 002" , "Meldung 003"
   Data "Meldung 004" , "Meldung 005"
   Data "Meldung 006" , "Meldung 007"
   Data "Meldung 008" , "Meldung 009"

'Ende

'###############################################################################